@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 test "x$srcdir" = x && srcdir=.
 test "x$builddir" = x && builddir=.
@@ -15,13 +15,18 @@ fi
 IFS=:
 for f in "$@"; do
 	echo "Running tests in $f"
-	while read fontfile unicodes glyphs_expected; do
+	while read fontfile options unicodes glyphs_expected; do
 		echo "Testing $fontfile:$unicodes"
-		glyphs=`$srcdir/hb-unicode-encode "$unicodes" | $hb_shape "$srcdir/$fontfile"`
+		glyphs=`$srcdir/hb-unicode-encode "$unicodes" | $hb_shape $options "$srcdir/$fontfile"`
+		if test $? != 0; then
+			echo "hb-shape failed." >&2
+			fails=$((fails+1))
+			continue
+		fi
 		if ! test "x$glyphs" = "x$glyphs_expected"; then
 			echo "Actual:   $glyphs" >&2
 			echo "Expected: $glyphs_expected" >&2
-			let fails=$fails+1
+			fails=$((fails+1))
 		fi
 	done < "$f"
 done
